/*----------------------------------------------------------------------------------------------------

--

-- File name	 : SDRAM_CTRLR.V

--					 :

-- Title			 : SDRAM CONTROLLER

--					 :

-- Purpose		 : To control Toshiba SDRAM

--					 :

-- IncludeFiles :

--					 :

-- Created On	 : August 1, 2001

--					 :

-- Comments		 : Created for Micron MT48LC16M16A2-8E 66 MHz SDRAM

--					 : Assuming 66 MHz clock:

--					 :		Refresh counter = Tref/(Tclk * #-of-rows) = 520 will be rounded down to 512

--					 :		Idle Counter = 512

--					 :		Num of initialization refreshs = 8

--					 :		Mode Register = "00 0000 0010 0011"

--					 :		=> burst length = 8

--					 :		=> Sequential burst type

--					 :		=> CAS latency = 2

--					 :		=> Burst read, Burst write

--					 :

--					 :		Active to column command, Lrcd = Trcd/Tclk = 2 clk

--					 :		Active to precharge command, Lras = Tras/Tclk = 4 clks

--					 :		Precharge to Active command, Lrp = Trp/Tclk = 2 clk

--					 :		Write to precharge command, Ldpl = Tdpl/Tclk = 2 clk

--					 :		Active to Active command, Lrrd = Trrd/Tclk (different bank) = 2 clk

--					 :		Ref/Active to Ref/Active command, Lrc = Trc/Tclk (same bank) = 5 clks

--					 :		Mode Register to next command, Lrsa = Trsa/Tclk = 2 clk

--					 :		Exit self refresh to active command, Lsec = Txsr/Tclk = 6 clks

--					 :		Write Recovery Time, Lapw = Tclk + 7 ns = 2 clks

--

--              :   Modifications made to operate the Toshiba SDRAM in the Spartan 2 reference board

--					 :

-- Conventions  : - Port names are upper case.

--					 : - Internal signals and variables are lower case.

--					 : - Active low signals are identified with '_n' or '_N'.

--					 : - constants begin with lower case 'c', rest upper case

--					 : - states begin with lower case 's', rest upper case

--					 : - aliases begin with lower case 'a', rest upper case

--					 :

--					 : Tab setting = 3

--					 : Right Margin = 100

--					 :

-- Limitations  : For use in Xilinx FPGA

--					 :

-- Developers	 : Duncan Allen (Translation from VHDL design done by Majid Kalsi)

----------------------------------------------------------------------------------------------------

-- Revision History :

----------------------------------------------------------------------------------------------------

--	  Version No | Author	 | Changes Made:							| Mod. Date:

--		 v1.0		 | da			 | First write								| 01/8/01

--     v1.1     | da        | Modified for Toshiba SDRAM       | 01/09/01

----------------------------------------------------------------------------------------------------*/

`timescale 1 ns/1 ps



module SDRAM_CONTROLLER  (

	// host signals

	CLK,

	RESET,

	ADDR,

	REN,



	WEN,

	DVLD,

	TRANSFER,

	READY,

	SDRAM_WRITE_DATA,

   SDRAM_READ_DATA,

   BYTE_MASK,





	// SDRAM signals

	SD_ADDR,

	SD_BS,

//	SD_CLK,

	SD_CKE,

	SD_CS_N,

	SD_RAS_N,

	SD_CAS_N,

	SD_WE_N,

	SD_DQM,

	SD_DATA_IN,

   SD_DATA_OUT,

   SD_DATA_ENABLE

	);

   // synthesis syn_edif_bit_format = "%u<%i>"

   // synthesis syn_edif_scalar_format = "%u"

   // synthesis syn_black_box



input				CLK; 	// The system clock

input				RESET;	// The system reset

input [22:0]	ADDR;    	// The address   [22:21] = bank, [20:10] = row, [9:2] = column [1:0] = ignored

input				REN;	// Read Enable

input				WEN;	// Write Enable

output			DVLD;		// Indicates the data on the output is valid

output 			TRANSFER; 	// Indicates that the data is currently being written to the SDRAM

output			READY;		// Indicates that the SDRAM controller is ready

input	[31:0]	SDRAM_WRITE_DATA;		// Data to be written to the SDRAM

output [31:0]  SDRAM_READ_DATA;     // Data read from the SDRAM

input [3:0]    BYTE_MASK; // The data masks from the FIFO



output [10:0]	SD_ADDR; 	// The SDRAM address

output [1:0]   	SD_BS;		// The SDRAM bank select



output			SD_CKE;		// The SDRAM clock enable

output 			SD_CS_N;	// The SDRAM chip select

output 			SD_RAS_N;	// The SDRAM row address strobe

output        	SD_CAS_N;	// The SDRAM column address strobe\

output 			SD_WE_N;	// The SDRAM write enable

output [3:0]	SD_DQM;		// The SDRAM data masks

input  [31:0] 	SD_DATA_IN;    // The SDRAM data read from the SDRAM

output [31:0]  SD_DATA_OUT;  // The SDRAM data being written to the SDRAM

output         SD_DATA_ENABLE; // The tristate control for SD_DATA







endmodule



