//--------------------------------------------------------------------------------------------------

//					:

// File name	: PCI_BUS_MONITOR.V

//					:

// Title			: PCI Bus Monitor

//					:

// Comments		:

//					:

//					:

// Conventions : - Port names are 'UPPER' case.

//					: - Internal signals and variables are 'lower' case.

//					: - Active low signals are identified with '_n' or '_N'.

//					: - instances begin with lower case 'i', rest upper case

//					: - parameters begin with lower case 'p', rest upper case

//					: - tasks begin with lower case 't', rest upper case

//					:

//					: Tab setting = 3

//					: Right Margin = 100

//					:

// Limitations : For use in testbench only

//					:

// Developers  : Majid Kalsi

//					:

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//	  Version No | Author	 | Changes Made:															| Mod. Date:

//		 v1.0		 | mk			 | First write																| 00/00/00

//--------------------------------------------------------------------------------------------------





`timescale 1 ns/1 ps

`include "PCI_device_defines.v"





//-----------------------------------------------

//  Module declaration

//-----------------------------------------------

module PCI_MONITOR

(

   CLK,

   FRAME_N,

   IRDY_N,

   DEVSEL_N,

   TRDY_N,

   STOP_N,

   CBE,

   INIT_XFER,

   TARG_XFER,

   CYCLE

);





//-----------------------------------------------

//  Port declarations

//-----------------------------------------------

input  CLK;

input  FRAME_N;

input  IRDY_N;

input  DEVSEL_N;

input  TRDY_N;

input  STOP_N;

input  [3:0] CBE;

output [239:0] INIT_XFER;

output [239:0] TARG_XFER;

output [239:0] CYCLE;



//-----------------------------------------------

//  Variable declarations

//-----------------------------------------------

reg [(30*`SIZEOFCHAR - 1):0] INIT_XFER;

reg [(30*`SIZEOFCHAR - 1):0] TARG_XFER;

reg [(30*`SIZEOFCHAR - 1):0] CYCLE;

reg mabort;







//-----------------------------------------------

//  INITIAL definition

//-----------------------------------------------

initial

begin

   INIT_XFER = "IDLE";

   TARG_XFER = "IDLE";

   CYCLE = "IDLE";

   mabort = 0;



//-----------------------------------------------

end  // End INITIAL Block

//-----------------------------------------------





//-----------------------------------------------

// PCI Target Transfer State

//-----------------------------------------------

always @(DEVSEL_N or TRDY_N or STOP_N or mabort)

begin

   if(mabort)

      TARG_XFER = "MABORT";

   else

      case({DEVSEL_N, TRDY_N, STOP_N})

         3'b000 :  TARG_XFER = "DWD";

         3'b001 :  TARG_XFER = "NORMAL";

         3'b010 :  TARG_XFER = "DWOD";

         3'b011 :  TARG_XFER = "WAIT";

         3'b110 :  TARG_XFER = "TABORT";

         default : TARG_XFER = "IDLE";

      endcase

end





//-----------------------------------------------

// PCI Initiator Transfer State

//-----------------------------------------------

always @(FRAME_N or IRDY_N)

begin

   case({FRAME_N, IRDY_N})

      2'b00 :   INIT_XFER = "PROCEED";

      2'b01 :   INIT_XFER = "WAIT";

      2'b10 :   INIT_XFER = "FINISH";

      default : INIT_XFER = "IDLE";

   endcase

end





//-----------------------------------------------

// PCI Cycle Monitor

//-----------------------------------------------

always @(negedge FRAME_N)

begin

   // Wait until rising edge of CLK

   @(posedge CLK);



   // Determine cycle type

   case(CBE)

      `INT_ACK :  CYCLE = "INT_ACK";

      `SPECIAL :  CYCLE = "SPECIAL";

      `IO_RD   :  CYCLE = "IO_RD";

      `IO_WR   :  CYCLE = "IO_WR";

      `MEM_RD  :  CYCLE = "MEM_RD";

      `MEM_WR  :  CYCLE = "MEM_WR";

      `CFG_RD  :  CYCLE = "CFG_RD";

      `CFG_WR  :  CYCLE = "CFG_WR";

      `MRM     :  CYCLE = "MRM";

      `DAC     :  CYCLE = "DAC";

      `MRL     :  CYCLE = "MRL";

      `MWI     :  CYCLE = "MWI";

      default  :

         begin

   			$display($time, " ns [PCI MONITOR] ERROR:  Unknown PCI Cycle");

         	CYCLE = "???";

         	$stop(2);

			end

   endcase



   // Wait until end of cycle

   while((INIT_XFER != "IDLE") || ((TARG_XFER != "IDLE") && (TARG_XFER != "MABORT")))

   	@(posedge CLK);



   CYCLE = "IDLE";

end



//-----------------------------------------------

// PCI Master Abort Monitor

//-----------------------------------------------

always @(negedge FRAME_N)

begin : MABORT_CHK

   integer i;



   // Wait until rising edge of CLK

   mabort = 0;

   @(posedge CLK);



   // Count number of clocks

   i = 0;

   while((i < 4) && (DEVSEL_N == 1)) begin

   	@(posedge CLK);

      i = i + 1;

   end



   if((i == 4) && (CYCLE != "SPECIAL")) begin

      mabort = 1;

   	$display($time, " ns [PCI MONITOR] ERROR: Master Abort");

      $stop(2);

   	@(posedge CLK);

      mabort = 0;

   end

end





endmodule